var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

import { mount } from 'enzyme';
import 'jest-enzyme';
import React from 'react';
import { shapeWith } from './../';
import { getPath, typeOf } from './../util';

describe('reshep > shapeWith', function () {
  test('Throws a TypeError if pathMap is not an object.', function () {
    var errType = function errType() {
      return TypeError;
    };
    var errMsg = function errMsg(type) {
      return 'Expected `pathMap` to be an object. ' + ('`pathMap` is type ' + type + ' instead.');
    };

    var testErr = function testErr(x) {
      expect(function () {
        return shapeWith(x);
      }).toThrow(errType());
      expect(function () {
        return shapeWith(x);
      }).toThrow(errMsg(typeOf(x)));
    };

    ['foo', 1, null, undefined, []].forEach(testErr);
  });

  test('Returns a higher-order component.', function () {
    var Base = function Base() {
      return React.createElement('p', {}, 'foo');
    };

    var HOC = shapeWith({})(Base);

    var wrapped = mount(React.createElement(HOC, {}));

    expect(wrapped).toContainReact(React.createElement(Base));
  });

  test('Component maps deeply nested props for each entry in `pathMap`.', function () {
    var props = {
      a: { b: { c: 1 } },
      d: { e: 2 },
      f: 3
    };

    var pathMap = {
      'a.b.c': 'a',
      'd.e': 'b.c',
      'f': 'd.e.f'
    };

    var Base = jest.fn();
    Base.mockReturnValue(null);

    var HOC = shapeWith(pathMap)(Base);

    mount(React.createElement(HOC, props));

    var baseProps = Base.mock.calls[0][0];

    Object.keys(pathMap).map(function (k) {
      return [k, pathMap[k]];
    }).map(function (paths) {
      return paths.map(function (p) {
        return p.split('.');
      });
    }).map(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          from = _ref2[0],
          to = _ref2[1];

      return [getPath(from, props), getPath(to, baseProps)];
    }).forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          from = _ref4[0],
          to = _ref4[1];

      return expect(from).toEqual(to);
    });
  });

  test('Component merges owner props and mapped props.', function () {
    var props = {
      a: { b: { c: 1 } },
      d: { e: 2 },
      f: 3
    };

    var pathMap = {
      'a.b.c': 'g',
      'd.e': 'h.i',
      'f': 'j.k.l'
    };

    var Base = jest.fn();
    Base.mockReturnValue(null);

    var HOC = shapeWith(pathMap)(Base);

    mount(React.createElement(HOC, props));

    var baseProps = Base.mock.calls[0][0];

    var expected = _extends({}, props, {
      g: 1,
      h: { i: 2 },
      j: { k: { l: 3 } }
    });

    expect(baseProps).toEqual(expected);
  });

  test('Owner props are overwritten by mapped props.', function () {
    var props = {
      a: { b: { c: 1 } },
      d: { e: 2 },
      f: 3
    };

    var pathMap = {
      'a.b.c': 'a',
      'f': 'd.e'
    };

    var Base = jest.fn();
    Base.mockReturnValue(null);

    var HOC = shapeWith(pathMap)(Base);

    mount(React.createElement(HOC, props));

    var baseProps = Base.mock.calls[0][0];

    var expected = {
      a: 1,
      d: { e: 3 },
      f: 3
    };

    expect(baseProps).toEqual(expected);
  });
});